//
//  AppDelegate.m
//  CoreDataLibraryApp
//
//  Created by Duncan Groenewald on 11/01/2014.
//  Copyright (c) 2014 OSSH Pty Ltd. All rights reserved.
//

#import "AppDelegate.h"
#import "OSCDStackManager.h"
#import "OpeningViewController.h"

@implementation AppDelegate

-(id)init {
    self = [super init];
    if (self)
    {
        [[OSCDStackManager sharedManager] setUbiquityContainerKey:UBIQUITY_CONTAINER_KEY];
        [[OSCDStackManager sharedManager] setUbiquityIDToken:UBID_TOKEN];
        [[OSCDStackManager sharedManager] setCloudPreferenceKey:COMPANY_ID "." APP_NAME "." CLOUD_PREFERENCE_KEY];
        [[OSCDStackManager sharedManager] setCloudPreferenceSet:COMPANY_ID "." APP_NAME "." CLOUD_PREFERENCE_SET];
        [[OSCDStackManager sharedManager] setMakeBackupPreferenceKey:COMPANY_ID "." APP_NAME "." BACKUP_PREFERENCE_KEY];
        [[OSCDStackManager sharedManager] setLoadInBackgroundPreferenceKey:COMPANY_ID "." APP_NAME "." BACKGROUNDLOAD_PREFERENCE_KEY];
    }
    return self;
}

- (BOOL)application:(UIApplication *)application didFinishLaunchingWithOptions:(NSDictionary *)launchOptions
{
    FLOG(@"didFinishLaunchingWithOptions called");
    
    //Simulate a long job here - such as migrating a persistentStore
    FLOG(@" starting long job (1 seconds) on main thread");
    sleep(1);
    FLOG(@" finished long job on main thread");
    
    // Set the version number in the Settings App settings page for this App
    [[OSCDStackManager sharedManager] setVersion];

    [[OSCDStackManager sharedManager] checkUserICloudPreferenceAndSetupIfNecessary];
    FLOG(@"checkUserICloudPreferenceAndSetupIfNecessary done.");
    
    // Override point for customization after application launch.
    if ([[UIDevice currentDevice] userInterfaceIdiom] == UIUserInterfaceIdiomPad) {
        
       UISplitViewController *splitViewController = (UISplitViewController *)self.window.rootViewController;
        UINavigationController *navigationController = [splitViewController.viewControllers lastObject];
        //splitViewController.delegate = (id)navigationController.topViewController;
        
        //UINavigationController *masterNavigationController = splitViewController.viewControllers[0];
        OpeningViewController *controller = (OpeningViewController *)navigationController.topViewController;
        //controller.managedObjectContext = [[OSCDStackManager sharedManager] managedObjectContext];
        self.detailViewManager = [[DetailViewManager alloc] init];
        self.detailViewManager.splitViewController = splitViewController;
        self.detailViewManager.detailViewController = (id)navigationController.topViewController;
        self.detailViewManager.openingViewController = controller;
        splitViewController.delegate = self.detailViewManager;

    } else {
        //UINavigationController *navigationController = (UINavigationController *)self.window.rootViewController;
        //CompanyViewController *controller = (CompanyViewController *)navigationController.topViewController;
        //controller.managedObjectContext = [[OSCDStackManager sharedManager] managedObjectContext];
    }
    
    //[[OSCDStackManager sharedManager] loadDataInBackground];
    
    FLOG(@"didFinishLaunchingWithOptions done.");
    
    // Add at end of application:didFinishLaunchingWithOptions
    NSURL *url = (NSURL *)[launchOptions valueForKey:UIApplicationLaunchOptionsURLKey];
    if (url != nil && [url isFileURL]) {
        [self handleOpenURL:url];
    }

    return YES;
}
- (BOOL)application:(UIApplication *)application openURL:(NSURL *)url sourceApplication:(NSString *)sourceApplication annotation:(id)annotation {
    FLOG(@" called");
    if (url != nil && [url isFileURL]) {
        [self handleOpenURL:url];
    }
    return YES;
}
// Add new method
-(BOOL) application:(UIApplication *)application handleOpenURL:(NSURL *)url {
    FLOG(@" called");
    if (url != nil && [url isFileURL]) {
        [self handleOpenURL:url];
    }
    return YES;
    
}
// New method
- (void)handleOpenURL:(NSURL *)url {
    FLOG(@" called");
    [[OSCDStackManager sharedManager] saveFile:(NSURL*)url];
    
}
- (void)applicationWillResignActive:(UIApplication *)application
{
    // Sent when the application is about to move from active to inactive state. This can occur for certain types of temporary interruptions (such as an incoming phone call or SMS message) or when the user quits the application and it begins the transition to the background state.
    // Use this method to pause ongoing tasks, disable timers, and throttle down OpenGL ES frame rates. Games should use this method to pause the game.
}

- (void)applicationDidEnterBackground:(UIApplication *)application
{
    FLOG(@"applicationDidEnterBackground: called");
    // Use this method to release shared resources, save user data, invalidate timers, and store enough application state information to restore your application to its current state in case it is terminated later. 
    // If your application supports background execution, this method is called instead of applicationWillTerminate: when the user quits.
    [[OSCDStackManager sharedManager] saveContext];
}

- (void)applicationWillEnterForeground:(UIApplication *)application
{
    FLOG(@"applicationWillEnterForeground: called");
    // Called as part of the transition from the background to the inactive state; here you can undo many of the changes made on entering the background.
    [[OSCDStackManager sharedManager] performApplicationWillEnterForegroundCheck];
    //[[OSCDStackManager sharedManager] loadDataInBackground];

}

- (void)applicationDidBecomeActive:(UIApplication *)application
{
    // Restart any tasks that were paused (or not yet started) while the application was inactive. If the application was previously in the background, optionally refresh the user interface.
}

- (void)applicationWillTerminate:(UIApplication *)application
{
    FLOG(@"applicationWillTerminate: called");
    // Saves changes in the application's managed object context before the application terminates.
    [[OSCDStackManager sharedManager] saveContext];
}




@end
