//
//  BackupsTableViewController.m
//  CoreDataLibraryApp
//
//  Created by Duncan Groenewald on 14/02/2014.
//  Copyright (c) 2014 OSSH Pty Ltd. All rights reserved.
//

#import "BackupsTableViewController.h"

@interface BackupsTableViewController ()

@end

@implementation BackupsTableViewController

- (id)initWithStyle:(UITableViewStyle)style
{
    self = [super initWithStyle:style];
    if (self) {
        // Custom initialization
    }
    return self;
}

- (void)viewDidLoad
{
    [super viewDidLoad];
    self.title = @"Backups";
    _localFileList = [[OSCDStackManager sharedManager] listAllLocalBackupDocuments];
    _icloudFileList = [[OSCDStackManager sharedManager] listAllICloudBackupDocuments];
    
    // Uncomment the following line to preserve selection between presentations.
    // self.clearsSelectionOnViewWillAppear = NO;
    
    // Uncomment the following line to display an Edit button in the navigation bar for this view controller.
    // self.navigationItem.rightBarButtonItem = self.editButtonItem;
    
    UIRefreshControl *refresh = [[UIRefreshControl alloc] init];
    
    refresh.attributedTitle = [[NSAttributedString alloc] initWithString:@"Pull to refresh"];
    
    [refresh addTarget:self action:@selector(refresh)
     
      forControlEvents:UIControlEventValueChanged];
    
    self.refreshControl = refresh;
    
    UIBarButtonItem *addButton = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemAdd target:self action:@selector(newBackup)];
    
    [self.navigationItem setRightBarButtonItem:addButton animated:YES];
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(filesChanged) name:OSFilesUpdatedNotification object:nil];
    
}
- (void)stopRefresh {
    
    [self.refreshControl endRefreshing];
    
}
- (void)refresh {
    //FLOG(@" called");
    [self reload];
    [self performSelector:@selector(stopRefresh) withObject:nil afterDelay:0.5];
}
- (void)filesChanged {
    FLOG(@" called");
    [self reload];
}
- (void)reload {
    _localFileList = [[OSCDStackManager sharedManager] listAllLocalBackupDocuments];
    _icloudFileList = [[OSCDStackManager sharedManager] listAllICloudBackupDocuments];
    [self.tableView reloadData];
}
- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}
- (void)newBackup
{
    FLOG(@" called");
    [[OSCDStackManager sharedManager] asynchronousTaskWithCompletion:^{
        FLOG(@" asynch backup completed");
        [self reload];
    }];
}
#pragma mark - Table view data source

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
    // Return the number of sections.
    return 2;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    FLOG(@" called");
    // Return the number of rows in the section.
    if (section == 0) // Local
        return [_localFileList count];
    else
        return [_icloudFileList count];
}
- (NSString *)tableView:(UITableView *)tableView titleForHeaderInSection:(NSInteger)section {
    //FLOG(@" called");
    if (section == 0)
        return @"Local";
    else
        return @"iCloud";
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    FLOG(@" called");
    static NSString *iCloudTableIdentifier = @"iCloudFileTableViewCell";
    UITableViewCell *cell;
    
    // Configure the cell...
    if (indexPath.section == 0) { // Local
        cell = [tableView dequeueReusableCellWithIdentifier:@"Cell" forIndexPath:indexPath];
        
        cell.textLabel.text = [[_localFileList objectAtIndex:indexPath.item] lastPathComponent];
        cell.detailTextLabel.text = [self timestamp:[_localFileList objectAtIndex:indexPath.item]];
        
    } else {                      // iCloud
        
        //FLOG(@" tableView:cellForRowAtIndexPath:");
        ICloudFileTableViewCell *tcell = (ICloudFileTableViewCell *)[tableView dequeueReusableCellWithIdentifier:iCloudTableIdentifier];
        if (tcell == nil)
        {
            //FLOG(@" cell is nil so create one");
            NSArray *nib = [[NSBundle mainBundle] loadNibNamed:@"ICloudFileTableViewCell" owner:self options:nil];
            //FLOG(@" nib is %@", nib);
            tcell = [nib objectAtIndex:0];
        }
        tcell.textLabel.text = [[_icloudFileList objectAtIndex:indexPath.item] valueForKey:@"filename"];
        tcell.detailTextLabel.text = [self timestamp:[[_icloudFileList objectAtIndex:indexPath.item] valueForKey:@"url"]];
        tcell.computerTextLabel.text = [[_icloudFileList objectAtIndex:indexPath.item] valueForKey:@"nameOfSavingComputer"];
        NSString *str = [[_icloudFileList objectAtIndex:indexPath.item] valueForKey:@"downloadStatus"];
        
        if ([str isEqualToString:NSMetadataUbiquitousItemDownloadingStatusCurrent]) {
            FLOG(@" XXX filename is %@", [[_icloudFileList objectAtIndex:indexPath.item] valueForKey:@"filename"]);
            FLOG(@"      download status is %@", str);
            FLOG(@"      download statusVar is %@", NSMetadataUbiquitousItemDownloadingStatusCurrent);
            FLOG(@"      download progress is 1");
            tcell.progressIndicator.progress = 1.0;
        } else {
            float percent = [[[_icloudFileList objectAtIndex:indexPath.item] valueForKey:@"percentDownloaded"] floatValue];
            FLOG(@" XXX filename is %@", [[_icloudFileList objectAtIndex:indexPath.item] valueForKey:@"filename"]);
            FLOG(@"      download status is %@", str);
            FLOG(@"      download statusVar is %@", NSMetadataUbiquitousItemDownloadingStatusCurrent);
            FLOG(@"     download progress is %f", percent);
            
            tcell.progressIndicator.progress = percent/100.0;
        }
        cell = tcell;
    }
    
    // Create and initialize a long press gesture
    UILongPressGestureRecognizer *lpr = [[UILongPressGestureRecognizer alloc]
                                         initWithTarget:self action:@selector(respondToLongPressGesture:)];
    lpr.minimumPressDuration = 1.0;
    [cell addGestureRecognizer:lpr];
    
    cell.accessoryType = UITableViewCellAccessoryNone;
    
    return cell;
}


/*
 // Override to support conditional editing of the table view.
 - (BOOL)tableView:(UITableView *)tableView canEditRowAtIndexPath:(NSIndexPath *)indexPath
 {
 // Return NO if you do not want the specified item to be editable.
 return YES;
 }
 */


// Override to support editing the table view.
- (void)tableView:(UITableView *)tableView commitEditingStyle:(UITableViewCellEditingStyle)editingStyle forRowAtIndexPath:(NSIndexPath *)indexPath
{
    if (editingStyle == UITableViewCellEditingStyleDelete) {
        // Delete the row from the data source
        if (indexPath.section == 0) {
            FLOG(@" deleting %@", [_localFileList objectAtIndex:indexPath.item]);
            //dispatch_sync(dispatch_get_main_queue(), ^{
            [[OSCDStackManager sharedManager] deleteBackupFile:[_localFileList objectAtIndex:indexPath.item]];
            [self reload];
            //});
        }
        else {
            FLOG(@" deleting %@", [[_icloudFileList objectAtIndex:indexPath.item] valueForKey:@"filename"]);
            //dispatch_sync(dispatch_get_main_queue(), ^{
            [[OSCDStackManager sharedManager] deleteBackupFile:[[_icloudFileList objectAtIndex:indexPath.item] valueForKey:@"fileuRL"]];
            [self reload];
            //});
        }
        
        //[tableView deleteRowsAtIndexPaths:@[indexPath] withRowAnimation:UITableViewRowAnimationFade];
    } else if (editingStyle == UITableViewCellEditingStyleInsert) {
        // Create a new instance of the appropriate class, insert it into the array, and add a new row to the table view
        dispatch_sync(dispatch_get_main_queue(), ^{
            [[OSCDStackManager sharedManager] backupCurrentStore];
            [self reload];
        });
        
    }
}


-(NSString*)timestamp:(NSURL*)fileURL {
    return [[[[fileURL URLByDeletingPathExtension] lastPathComponent] componentsSeparatedByString:@"_Backup_"] lastObject];
}


- (void)respondToLongPressGesture:(UIGestureRecognizer *)gestureRecognizer
{
    if (gestureRecognizer.state == UIGestureRecognizerStateBegan) {
        UITableViewCell *cell = (UITableViewCell *)gestureRecognizer.view;
        NSIndexPath* indexPath = [self.tableView indexPathForCell:cell];
        FLOG(@"respondToLongPressGesture called for row %ld", (long)indexPath.row);
        // Show a popup menu here
        [self showCellMenu:cell];
    }
}
-(void)showCellMenu:(UITableViewCell *)cell
{
    FLOG(@"showCellMenu called");
    
    // Bail out because this is not allowed on iPhone
    //if ([[UIDevice currentDevice] userInterfaceIdiom] == UIUserInterfaceIdiomPhone)
    //    return;
    _cellMenuPickerPopover = nil;
    CGRect rect = cell.frame;
    rect.size.width -= 100;
    
    NSIndexPath* indexPath = [self.tableView indexPathForCell:cell];
    
    if (indexPath.section == 0) {
        _cellMenuPicker = [[CellMenuPickerViewController alloc] initWithStyle:UITableViewStylePlain type:[[self.tableView indexPathForCell:cell] section]];
    } else {
        _cellMenuPicker = [[CellMenuPickerViewController alloc] initWithStyle:UITableViewStylePlain
                                                                      forCell:cell
                                                                         type:[[self.tableView indexPathForCell:cell] section]
                                                                         file:[_icloudFileList objectAtIndex:indexPath.item]];
    }

    //Set this VC as the delegate.
    _cellMenuPicker.delegate = self;
    
    [_cellMenuPicker setCell:cell];
    
    if (_cellMenuPickerPopover == nil) {
        //The color picker popover is not showing. Show it.
        _cellMenuPickerPopover = [[UIPopoverController alloc] initWithContentViewController:_cellMenuPicker];
        
        _cellMenuPickerPopover.popoverContentSize = [_cellMenuPicker getSize ] ;
        
        [_cellMenuPickerPopover presentPopoverFromRect:rect inView:self.view permittedArrowDirections:UIPopoverArrowDirectionLeft animated:YES ];
        
    } else {
        [_cellMenuPickerPopover presentPopoverFromRect:rect inView:self.view permittedArrowDirections:UIPopoverArrowDirectionLeft animated:YES ];
    }
}
-(void)selectedCellMenu:(NSString*)menu forCell:(UITableViewCell *)cell
{
    FLOG(@" called with %@ for %@", menu, cell.textLabel.text);
    NSIndexPath* indexPath = [self.tableView indexPathForCell:cell];
    NSURL *fileURL;
    if (indexPath.section == 0) {
        fileURL = [_localFileList objectAtIndex:indexPath.item];
    } else {
        fileURL = [[_icloudFileList objectAtIndex:indexPath.item] valueForKey:@"url"];
    }
    
    //FLOG(@" for row %d", indexPath.row);
    NSString *fileName = cell.textLabel.text;
    
    //Dismiss the popover if it's showing.
    if (_cellMenuPickerPopover) {
        [_cellMenuPickerPopover dismissPopoverAnimated:YES];
        _cellMenuPickerPopover = nil;
    }
    
    // Take the necessary Action
    if ([menu isEqualToString:CELLACTION_DELETE])
    {
        FLOG(@" delete item %@", fileName);
        [[OSCDStackManager sharedManager] deleteBackupFile:fileURL];
        [self reload];
        
    } else
        if ([menu isEqualToString:CELLACTION_COPYTOICLOUD])
        {
            FLOG(@" Copy to iCloud %@", fileName);
            [self copyToICloud:fileURL];
        } else
            if ([menu isEqualToString:CELLACTION_DOWNLOADFROMICLOUD])
            {
                FLOG(@" Download from iCloud %@", fileName);
                [self downloadFromICloud:[_icloudFileList objectAtIndex:indexPath.item]];
            } else
                if ([menu isEqualToString:CELLACTION_COPYFROMICLOUD])
                {
                    FLOG(@" Copy from iCloud %@", fileName);
                    [self copyFromICloud:[_icloudFileList objectAtIndex:indexPath.item]];
                } else
                if ([menu isEqualToString:CELLACTION_SENDEMAIL])
                {
                    FLOG(@" Send using Email %@", fileName);
                    [self sendUsingEmail:fileURL];
                } else
                    if ([menu isEqualToString:CELLACTION_RESTORE])
                    {
                        FLOG(@" Restore from %@", fileName);
                        [self restore:fileURL];
                    }
}
- (void)copyToICloud:(NSURL*)fileURL
{
    FLOG(@" called");
    [[OSCDStackManager sharedManager] asynchronousCopyToICloudWithCompletion:fileURL completionHandler:^{
        FLOG(@" copyToICloud completed, waiting for iCloud sync");
        //[self reload];
    }];
}
- (void)copyFromICloud:(FileRepresentation*)file
{
    FLOG(@" called");
    [[OSCDStackManager sharedManager] asynchronousCopyFromICloudWithCompletion:file.url completionHandler:^{
        FLOG(@" copyFromICloud completed, waiting for iCloud sync");
        [self reload];
    }];
}
- (void)downloadFromICloud:(FileRepresentation*)file
{
    FLOG(@" called");
    // Start the download
    [[OSCDStackManager sharedManager] downloadFile:file];
}
- (void)sendUsingEmail:(NSURL*)fileURL
{
    FLOG(@" called");
    [self showEmail:fileURL];
}
- (void)restore:(NSURL*)fileURL
{
    FLOG(@" called");
    [[OSCDStackManager sharedManager] asynchronousRestoreFile:fileURL completionHandler:^{
        FLOG(@" restore completed, waiting for iCloud sync");
        [self reload];
    }];
}

- (void)showEmail:(NSURL*)fileURL {
    
    NSString *emailTitle = @"CoreDataLibraryApp Backup File";
    NSString *messageBody = @"Backup file is attached.";
    NSArray *toRecipients = [NSArray arrayWithObject:@"duncan.groenewald@ossh.com.au"];
    
    MFMailComposeViewController *mc = [[MFMailComposeViewController alloc] init];
    mc.mailComposeDelegate = self;
    [mc setSubject:emailTitle];
    [mc setMessageBody:messageBody isHTML:NO];
    [mc setToRecipients:toRecipients];
    
    // Determine the file name and extension
    NSString *filename = [fileURL lastPathComponent];
    NSString *extension = [fileURL pathExtension];
    
    // Get the resource path and read the file using NSData
    NSData *fileData = [NSData dataWithContentsOfFile:[fileURL path]];
    
    // Determine the MIME type
    NSString *mimeType;
    if ([extension isEqualToString:@"jpg"]) {
        mimeType = @"image/jpeg";
    } else if ([extension isEqualToString:@"png"]) {
        mimeType = @"image/png";
    } else if ([extension isEqualToString:@"doc"]) {
        mimeType = @"application/msword";
    } else if ([extension isEqualToString:@"ppt"]) {
        mimeType = @"application/vnd.ms-powerpoint";
    } else if ([extension isEqualToString:@"html"]) {
        mimeType = @"text/html";
    } else if ([extension isEqualToString:@"pdf"]) {
        mimeType = @"application/pdf";
    } else if ([extension isEqualToString:@"sqlite"]) {
        mimeType = @"application/sqlite";
    } else
        mimeType = @"application/sqlite";
    
    FLOG(@" filename is %@", filename);
    FLOG(@" mimeType is %@", mimeType);
    // Add attachment
    [mc addAttachmentData:fileData mimeType:mimeType fileName:filename];
    
    // Present mail view controller on screen
    [self presentViewController:mc animated:YES completion:NULL];
    
}

- (void) mailComposeController:(MFMailComposeViewController *)controller didFinishWithResult:(MFMailComposeResult)result error:(NSError *)error
{
    switch (result)
    {
        case MFMailComposeResultCancelled:
            NSLog(@"Mail cancelled");
            break;
        case MFMailComposeResultSaved:
            NSLog(@"Mail saved");
            break;
        case MFMailComposeResultSent:
            NSLog(@"Mail sent");
            break;
        case MFMailComposeResultFailed:
            NSLog(@"Mail sent failure: %@", [error localizedDescription]);
            break;
        default:
            break;
    }
    
    // Close the Mail Interface
    [self dismissViewControllerAnimated:YES completion:NULL];
}
@end
