//
//  CellMenuPickerViewController.m
//  iProjectiOS
//
//  Created by Duncan Groenewald on 23/07/13.
//  Copyright (c) 2013 Duncan Groenewald. All rights reserved.
//

#import "CellMenuPickerViewController.h"
#import "Macros.h"


@interface CellMenuPickerViewController ()

@end

@implementation CellMenuPickerViewController
- (id)initWithStyle:(UITableViewStyle)style type:(NSInteger)type
{
    self = [super initWithStyle:style];
    if (self) {
        // Custom initialization
        //Initialize the array
        _itemTypes = [[NSMutableArray alloc] init];
        //Set up the array of colors.
        [_itemTypes addObject:CELLACTION_DELETE];
        if (type ==0) {
            [_itemTypes addObject:CELLACTION_COPYTOICLOUD];
            [_itemTypes addObject:CELLACTION_SENDEMAIL];
            [_itemTypes addObject:CELLACTION_RESTORE];
        }
        else
            [_itemTypes addObject:CELLACTION_COPYFROMICLOUD];

        
        //Make row selections persist.
        self.clearsSelectionOnViewWillAppear = NO;

        //Set the property to tell the popover container how big this view will be.
        self.preferredContentSize = [self getSize ];
    }
    return self;
}
- (id)initWithStyle:(UITableViewStyle)style forCell:(UITableViewCell*)cell type:(NSInteger)type file:(FileRepresentation*)file
{
    FLOG(@" called for %@, %@", file.filename, file.isDownloaded);
    self = [super initWithStyle:style];
    if (self) {
        // Custom initialization
        //Initialize the array
        _itemTypes = [[NSMutableArray alloc] init];
        _cell = cell;
        
        //Set up the array of colors.
        [_itemTypes addObject:CELLACTION_DELETE];
        if (type ==0) {
            FLOG(@" section is Local");
            [_itemTypes addObject:CELLACTION_COPYTOICLOUD];
            [_itemTypes addObject:CELLACTION_SENDEMAIL];
            [_itemTypes addObject:CELLACTION_RESTORE];
        }
        else {
            FLOG(@" section is iCloud");
            if (file.isDownloaded.boolValue) {
                FLOG(@" file is downloaded");
                [_itemTypes addObject:CELLACTION_SENDEMAIL];
                [_itemTypes addObject:CELLACTION_RESTORE];
                [_itemTypes addObject:CELLACTION_COPYFROMICLOUD];
            } else {
                FLOG(@" file is NOT downloaded");
                [_itemTypes addObject:CELLACTION_DOWNLOADFROMICLOUD];
            }
        }

        //Make row selections persist.
        self.clearsSelectionOnViewWillAppear = NO;
        
        //FLOG(@"initWithStyle called for Cell %@", cell);
        //Set the property to tell the popover container how big this view will be.
        self.preferredContentSize = [self getSize ];
    }
    return self;
}
- (CGSize)getSize {
   
    //Calculate how tall the view should be by multiplying
    //the individual row height by the total number of rows.
    NSInteger rowsCount = [_itemTypes count];
    NSInteger singleRowHeight = [self.tableView.delegate tableView:self.tableView
                                           heightForRowAtIndexPath:[NSIndexPath indexPathForRow:0 inSection:0]];
    NSInteger totalRowsHeight = rowsCount * singleRowHeight;
    //NSLog(@"CellMenu Picker rows %d height %d", rowsCount, singleRowHeight);
    
    //Calculate how wide the view should be by finding how
    //wide each string is expected to be
    CGFloat largestLabelWidth = 0;
    for (NSString *itemType in _itemTypes) {
        //Checks size of text using the default font for UITableViewCell's textLabel.
        NSMutableDictionary *dict = [[NSMutableDictionary alloc] init];
        [dict setValue:[UIFont boldSystemFontOfSize:20.0f] forKey:NSFontAttributeName];
        CGSize labelSize = [itemType sizeWithAttributes:dict];
        if (labelSize.width > largestLabelWidth) {
            largestLabelWidth = labelSize.width;
        }
    }
    
    //Add a little padding to the width
    CGFloat popoverWidth = largestLabelWidth + 80;
    
    //NSLog(@"CellMenu Picker width %0.1f", popoverWidth);
    
    //Set the property to tell the popover container how big this view will be.
    return CGSizeMake(popoverWidth, totalRowsHeight);
}

- (void)viewDidLoad
{
    [super viewDidLoad];
    
    // Uncomment the following line to preserve selection between presentations.
    // self.clearsSelectionOnViewWillAppear = NO;
    
    // Uncomment the following line to display an Edit button in the navigation bar for this view controller.
    // self.navigationItem.rightBarButtonItem = self.editButtonItem;
}

- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

#pragma mark - Table view data source

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
    // Return the number of sections.
    return 1;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    // Return the number of rows in the section.
    return [_itemTypes count];
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    static NSString *CellIdentifier = @"Cell";
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:CellIdentifier];
    if (cell == nil) {
        cell = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:CellIdentifier];
    }
    
    // Configure the cell...
    cell.textLabel.text = [_itemTypes objectAtIndex:indexPath.row];
    [cell setSelectionStyle:UITableViewCellSelectionStyleNone];
    return cell;
}


#pragma mark - Table view delegate
- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
    
    NSString *selectedMenu = [_itemTypes objectAtIndex:indexPath.row];
    
    
    //Notify the delegate if it exists.
    if (_delegate != nil) {
        [_delegate selectedCellMenu:selectedMenu forCell:_cell];
    }
}

@end
