//
//  DetailViewController.m
//  CoreDataLibraryApp
//
//  Created by Duncan Groenewald on 11/01/2014.
//  Copyright (c) 2014 OSSH Pty Ltd. All rights reserved.
//

#import "CompanyDetailViewController.h"
#import "OSCDStackManager.h"
#import "ImageTableViewCell.h"

@interface CompanyDetailViewController ()
@property (strong, nonatomic) UIPopoverController *masterPopoverController;
- (void)configureView;
@end

@implementation CompanyDetailViewController


- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil
{
    self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
    if (self) {
        // Custom initialization
        self.navigationBar.topItem.title = NSLocalizedString(@"", @"");
    }
    return self;
}
- (void)setNavigationPaneBarButtonItem:(UIBarButtonItem *)navigationPaneBarButtonItem
{
    FLOG(@" called with %@", navigationPaneBarButtonItem);

    if (navigationPaneBarButtonItem != _navigationPaneBarButtonItem) {
        // Add the popover button to the left navigation item.
        [self.navigationBar.topItem setLeftBarButtonItem:navigationPaneBarButtonItem
                                                animated:NO];
        
        _navigationPaneBarButtonItem = navigationPaneBarButtonItem;
    }
}

#pragma mark - Managing the detail item

- (void)setDetailItem:(id)newDetailItem
{
    if (_detailItem != newDetailItem) {
        _detailItem = newDetailItem;
        
        // Update the view.
        [self configureView];
    }

    if (self.masterPopoverController != nil) {
        [self.masterPopoverController dismissPopoverAnimated:YES];
    }        
}

- (void)configureView
{
    // Setup UI Controls
    [self.cancelButton setEnabled:NO];
    [self.cancelButton addTarget:self action:@selector(cancel) forControlEvents:UIControlEventTouchDown];
    [self.saveButton setEnabled:NO];
    [self.saveButton addTarget:self action:@selector(saveDetails) forControlEvents:UIControlEventTouchDown];
    
    // Set textField delegate
    self.detailDescriptionTextField.delegate = self;
    
    _fetchedResultsController = nil;
    _fetchedResultsControllerProj = nil;

    
    // Set the UITableView delegate
    self.tableView.delegate = self;
    self.tableView.dataSource = self;

    self.projTableView.delegate = self;
    self.projTableView.dataSource = self;

    // Display the objects details
    [self displayDetails];

}

- (void)displayDetails
{
    // Update the user interface for the detail item.
    
    if (self.detailItem) {
        self.detailDescriptionTextField.text = [[self.detailItem valueForKey:@"name"] description];
    }
    [self.tableView reloadData];
    [self.projTableView reloadData];
    
}

- (void)saveDetails
{
    // Save any changes in the UI components to the managedObject (can't cancel after this)
    
    if (self.detailItem) {
        
        [self.detailItem setValue:self.detailDescriptionTextField.text forKey:@"name"];
        
        NSError *error;
        if (![[self.detailItem managedObjectContext] save:&error]) {
            FLOG(@" error saving changes!");
            FLOG(@" error is %@, %@", error, error.userInfo);
        } else {
            FLOG(@" changes saved!");
            [self setEdited:NO];
        }
    }
    
}

- (void)cancel
{
    // Update the user interface with the original data
    
    [self displayDetails];
    
    [self setEdited:NO];
    
}

- (void)setEdited:(BOOL)edited
{
    _edited = edited;
    
        [self.cancelButton setEnabled:edited];
        [self.saveButton setEnabled:edited];
}

- (void)refreshUI {
    FLOG(@"refreshUI called");
    [self displayDetails];
}

- (void)viewDidLoad
{
    [super viewDidLoad];
	// Do any additional setup after loading the view, typically from a nib.
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(refreshUI) name:OSDataUpdatedNotification
                                               object:nil];

    if (floor(NSFoundationVersionNumber) <= NSFoundationVersionNumber_iOS_6_1) {
        // Load resources for iOS 6.1 or earlier
    } else {
        // Load resources for iOS 7 or later
        self.edgesForExtendedLayout = UIRectEdgeNone;
    }
    
    self.title = NSLocalizedString(@"Company Detail", @"Company Detail");
    [self configureView];
}

- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

#pragma mark - Split view

- (void)splitViewController:(UISplitViewController *)splitController willHideViewController:(UIViewController *)viewController withBarButtonItem:(UIBarButtonItem *)barButtonItem forPopoverController:(UIPopoverController *)popoverController
{
    barButtonItem.title = NSLocalizedString(@"Master", @"Master");
    [self.navigationItem setLeftBarButtonItem:barButtonItem animated:YES];
    self.masterPopoverController = popoverController;
}

- (void)splitViewController:(UISplitViewController *)splitController willShowViewController:(UIViewController *)viewController invalidatingBarButtonItem:(UIBarButtonItem *)barButtonItem
{
    // Called when the view is shown again in the split view, invalidating the button and popover controller.
    [self.navigationItem setLeftBarButtonItem:nil animated:YES];
    self.masterPopoverController = nil;
}

#pragma mark - TextFieldDelgate


- (BOOL)textField:(UITextField *)textField shouldChangeCharactersInRange:(NSRange)range replacementString:(NSString *)string
{
    [self setEdited:YES];
    return YES;
}

#pragma mark - Fetched results controller

- (NSFetchedResultsController *)fetchedResultsController
{
    if (_fetchedResultsController != nil) {
        return _fetchedResultsController;
    }
    
    if (self.detailItem == nil) return nil;
    
    NSFetchRequest *fetchRequest = [[NSFetchRequest alloc] init];
    // Edit the entity name as appropriate.
    NSEntityDescription *entity = [NSEntityDescription entityForName:@"Person" inManagedObjectContext:[self.detailItem managedObjectContext]];
    [fetchRequest setEntity:entity];
    
    // Set the batch size to a suitable number.
    [fetchRequest setFetchBatchSize:20];
    
    // Set the Predicate
    NSPredicate *predicate = [NSPredicate predicateWithFormat:@"employer == %@", self.detailItem];
    [fetchRequest setPredicate:predicate];
    
    // Edit the sort key as appropriate.
    NSSortDescriptor *sortDescriptor = [[NSSortDescriptor alloc] initWithKey:@"name" ascending:YES];
    NSArray *sortDescriptors = @[sortDescriptor];
    
    [fetchRequest setSortDescriptors:sortDescriptors];
    
    // Edit the section name key path and cache name if appropriate.
    // nil for section name key path means "no sections".
    NSFetchedResultsController *aFetchedResultsController = [[NSFetchedResultsController alloc] initWithFetchRequest:fetchRequest managedObjectContext:[self.detailItem managedObjectContext] sectionNameKeyPath:nil cacheName:nil];
    aFetchedResultsController.delegate = self;
    self.fetchedResultsController = aFetchedResultsController;
    
	NSError *error = nil;
	if (![self.fetchedResultsController performFetch:&error]) {
        // Replace this implementation with code to handle the error appropriately.
        // abort() causes the application to generate a crash log and terminate. You should not use this function in a shipping application, although it may be useful during development.
	    NSLog(@"Unresolved error %@, %@", error, [error userInfo]);
	    //abort();
	}
    
    return _fetchedResultsController;
}

- (NSFetchedResultsController *)fetchedResultsControllerProj
{
    if (_fetchedResultsControllerProj != nil) {
        return _fetchedResultsControllerProj;
    }
    
    if (self.detailItem == nil) return nil;
    
    NSFetchRequest *fetchRequest = [[NSFetchRequest alloc] init];
    // Edit the entity name as appropriate.
    NSEntityDescription *entity = [NSEntityDescription entityForName:@"ProjectType" inManagedObjectContext:[self.detailItem managedObjectContext]];
    [fetchRequest setEntity:entity];
    
    // Set the batch size to a suitable number.
    [fetchRequest setFetchBatchSize:20];
    
    // Edit the sort key as appropriate.
    NSSortDescriptor *sortDescriptor = [[NSSortDescriptor alloc] initWithKey:@"name" ascending:YES];
    NSArray *sortDescriptors = @[sortDescriptor];
    
    [fetchRequest setSortDescriptors:sortDescriptors];
    
    // Edit the section name key path and cache name if appropriate.
    // nil for section name key path means "no sections".
    NSFetchedResultsController *aFetchedResultsController = [[NSFetchedResultsController alloc] initWithFetchRequest:fetchRequest managedObjectContext:[self.detailItem managedObjectContext] sectionNameKeyPath:nil cacheName:nil];
    aFetchedResultsController.delegate = self;
    self.fetchedResultsControllerProj = aFetchedResultsController;
    
	NSError *error = nil;
	if (![self.fetchedResultsControllerProj performFetch:&error]) {
        // Replace this implementation with code to handle the error appropriately.
        // abort() causes the application to generate a crash log and terminate. You should not use this function in a shipping application, although it may be useful during development.
	    NSLog(@"Unresolved error %@, %@", error, [error userInfo]);
	    //abort();
	}
    
    return _fetchedResultsControllerProj;
}
- (void)controllerWillChangeContent:(NSFetchedResultsController *)controller
{
    if (controller == _fetchedResultsController)
        [self.tableView beginUpdates];
    else if (controller == _fetchedResultsControllerProj)
        [self.projTableView beginUpdates];
}

- (void)controller:(NSFetchedResultsController *)controller didChangeSection:(id <NSFetchedResultsSectionInfo>)sectionInfo
           atIndex:(NSUInteger)sectionIndex forChangeType:(NSFetchedResultsChangeType)type
{
    switch(type) {
        case NSFetchedResultsChangeInsert:
            [self.tableView insertSections:[NSIndexSet indexSetWithIndex:sectionIndex] withRowAnimation:UITableViewRowAnimationFade];
            break;
            
        case NSFetchedResultsChangeDelete:
            [self.tableView deleteSections:[NSIndexSet indexSetWithIndex:sectionIndex] withRowAnimation:UITableViewRowAnimationFade];
            break;
    }
}

- (void)controller:(NSFetchedResultsController *)controller didChangeObject:(id)anObject
       atIndexPath:(NSIndexPath *)indexPath forChangeType:(NSFetchedResultsChangeType)type
      newIndexPath:(NSIndexPath *)newIndexPath
{
    UITableView *tableView;
    
    if (controller == _fetchedResultsController)
        tableView = self.tableView;
     else if (controller == _fetchedResultsControllerProj)
         tableView = self.projTableView;
    
    switch(type) {
        case NSFetchedResultsChangeInsert:
            [tableView insertRowsAtIndexPaths:@[newIndexPath] withRowAnimation:UITableViewRowAnimationFade];
            break;
            
        case NSFetchedResultsChangeDelete:
            [tableView deleteRowsAtIndexPaths:@[indexPath] withRowAnimation:UITableViewRowAnimationFade];
            break;
            
        case NSFetchedResultsChangeUpdate:
            [self configureCell:controller cell:[tableView cellForRowAtIndexPath:indexPath] atIndexPath:indexPath];
            break;
            
        case NSFetchedResultsChangeMove:
            [tableView deleteRowsAtIndexPaths:@[indexPath] withRowAnimation:UITableViewRowAnimationFade];
            [tableView insertRowsAtIndexPaths:@[newIndexPath] withRowAnimation:UITableViewRowAnimationFade];
            break;
    }
}

- (void)controllerDidChangeContent:(NSFetchedResultsController *)controller
{
    if (controller == _fetchedResultsController)
        [self.tableView endUpdates];
    else if (controller == _fetchedResultsControllerProj)
        [self.projTableView endUpdates];

}

- (void)configureCell:(NSFetchedResultsController *)controller cell:(UITableViewCell *)cell atIndexPath:(NSIndexPath *)indexPath
{
    if (controller) {
        // If it's the Person controller
        if (controller == _fetchedResultsController) {
            cell.accessoryView = nil;
            NSManagedObject *object = [controller objectAtIndexPath:indexPath];
            cell.textLabel.text = [[object valueForKey:@"name"] description];
            cell.accessoryType = UITableViewCellAccessoryNone;
            
            //NSData *data = [object valueForKey:@"imageData"];
            UIImage *image = [object valueForKey:@"image"];
            cell.imageView.image = image;
            
        } else { // It must be the projectType controller
            cell.accessoryView = nil;
            NSManagedObject *object = [controller objectAtIndexPath:indexPath];
            cell.textLabel.text = [[object valueForKey:@"name"] description];
            cell.accessoryType = UITableViewCellAccessoryNone;
            
        }
        
        
    } else {
        cell.textLabel.text = @"";
        cell.accessoryType = UITableViewCellAccessoryNone;
    }
}

#pragma mark - Table View

// if fetchedResultsController == nil it means the database has not been initialised yet
// and so we just return a single row saying "Loading, please wait..."

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
    if (tableView == self.tableView) {
        // return 1 section of fetechedResultsController == nil
        if (self.fetchedResultsController == nil)
            return 1;
        else
            return [[self.fetchedResultsController sections] count];
    }
    if (tableView == self.projTableView) {
        // return 1 section of fetechedResultsController == nil
        if (self.fetchedResultsControllerProj == nil)
            return 1;
        else
            return [[self.fetchedResultsControllerProj sections] count];
    }
    return 1;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    if (tableView == self.tableView) {
        if (self.fetchedResultsController == nil) {
            return 1;
        } else {
            id <NSFetchedResultsSectionInfo> sectionInfo = [self.fetchedResultsController sections][section];
            return [sectionInfo numberOfObjects];
        }
    }
    if (tableView == self.projTableView) {
        if (self.fetchedResultsControllerProj == nil) {
            return 1;
        } else {
            id <NSFetchedResultsSectionInfo> sectionInfo = [self.fetchedResultsControllerProj sections][section];
            return [sectionInfo numberOfObjects];
        }
    }
    return 1;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    
    if (tableView == self.tableView) {
        
        static NSString *simpleTableIdentifier = @"ImageTableViewCell";
        
        ImageTableViewCell *cell = (ImageTableViewCell *)[tableView dequeueReusableCellWithIdentifier:simpleTableIdentifier];
        if (cell == nil)
        {
            NSArray *nib = [[NSBundle mainBundle] loadNibNamed:@"ImageTableViewCell" owner:self options:nil];
            cell = [nib objectAtIndex:0];
        }

        [self configureCell:_fetchedResultsController cell:cell atIndexPath:indexPath];
        return cell;
        
    } else if (tableView == self.projTableView) {
        UITableViewCell *cell;

        cell = [tableView dequeueReusableCellWithIdentifier:@"ProjectCell" forIndexPath:indexPath];
        [self configureCell:_fetchedResultsControllerProj cell:cell atIndexPath:indexPath];
        return cell;
    }
    
    UITableViewCell *cell;
        
    cell = [tableView dequeueReusableCellWithIdentifier:@"Cell" forIndexPath:indexPath];
    return cell;
}

- (BOOL)tableView:(UITableView *)tableView canEditRowAtIndexPath:(NSIndexPath *)indexPath
{
    if (tableView == self.tableView) {
        if (self.fetchedResultsController == nil) {
            return NO;
        } else {
            return YES;
        }
    } else if (tableView == self.projTableView) {
        if (self.fetchedResultsControllerProj == nil) {
            return NO;
        } else {
            return YES;
        }
    }
    return NO;
}

- (void)tableView:(UITableView *)tableView commitEditingStyle:(UITableViewCellEditingStyle)editingStyle forRowAtIndexPath:(NSIndexPath *)indexPath
{
    if (tableView == self.tableView) {
        if (self.fetchedResultsController == nil) {
        } else {
            if (editingStyle == UITableViewCellEditingStyleDelete) {
                NSManagedObjectContext *context = [self.fetchedResultsController managedObjectContext];
                // Oops we don't really want to delete the person, just set their employer attribute to NIL
                //[context deleteObject:[self.fetchedResultsController objectAtIndexPath:indexPath]];
                
                // FIX: set persons employer to nil
                [[self.fetchedResultsController objectAtIndexPath:indexPath] setValue:(nil) forKey:@"company"];
                
                NSError *error = nil;
                if (![context save:&error]) {
                    NSLog(@"Unresolved error %@, %@", error, [error userInfo]);
                }
            }
        }
    }
    if (tableView == self.projTableView) {
        if (self.fetchedResultsControllerProj == nil) {
        } else {
            if (editingStyle == UITableViewCellEditingStyleDelete) {
                NSManagedObjectContext *context = [self.fetchedResultsControllerProj managedObjectContext];
                [context deleteObject:[self.fetchedResultsControllerProj objectAtIndexPath:indexPath]];
                
                NSError *error = nil;
                if (![context save:&error]) {
                    NSLog(@"Unresolved error %@, %@", error, [error userInfo]);
                }
            }
        }
    }
}
- (NSIndexPath *)tableView:(UITableView *)tableView willSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    
    if (tableView == self.tableView) {
        // Don't allow user to select a row if Core Data stack is not set up yet.
        if (self.fetchedResultsController == nil) {
            NSLog(@"  fetchedResultsController == nil");
            return nil;
        }
    }
    if (tableView == self.projTableView) {
        // Don't allow user to select a row if Core Data stack is not set up yet.
        if (self.fetchedResultsControllerProj == nil) {
            NSLog(@"  fetchedResultsController == nil");
            return nil;
        }
    }
    // By default, allow row to be selected
    return indexPath;
}
@end


