//
//  MenuViewController.m
//  CoreDataLibraryApp
//
//  Created by Duncan Groenewald on 12/02/2014.
//  Copyright (c) 2014 OSSH Pty Ltd. All rights reserved.
//

#import "MenuViewController.h"
#import "CompanyViewController.h"
#import "PeopleViewController.h"
#import "BackupsTableViewController.h"
#import "BackgroundTaskViewController.h"

#define MENU_COMPANY_CONTACT    @"Companies & Contacts"
#define MENU_FINANCE            @"Finance"
#define MENU_QUALITY            @"Quality"
#define MENU_ADMIN              @"Administration"

#define SUBMENU_COMPANY          @"Companies"
#define SUBMENU_PEOPLE           @"People"
#define SUBMENU_CONTACTS         @"Contacts"
#define SUBMENU_PROJECTS         @"Projects"
#define SUBMENU_TASKS            @"Tasks"

#define SUBMENU_BACKUPS          @"Backups"
#define SUBMENU_BACKGROUND       @"Background Tasks"
#define SUBMENU_ICLOUD          @"iCloud"


@interface MenuViewController ()

@end

@implementation MenuViewController

- (id)initWithStyle:(UITableViewStyle)style
{
    //FLOG(@" called");
    
    self = [super initWithStyle:style];
    if (self) {
        // Custom initialization

    }
    return self;
}

- (void)viewDidLoad
{
    //FLOG(@" called");
    [super viewDidLoad];
    [self loadMenuData];
    _isLoaded = NO;
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(storeChanged) name:OSStoreChangeNotification
                                               object:nil];

    // Uncomment the following line to preserve selection between presentations.
    // self.clearsSelectionOnViewWillAppear = NO;
    
    // Uncomment the following line to display an Edit button in the navigation bar for this view controller.
    // self.navigationItem.rightBarButtonItem = self.editButtonItem;
    
    UIRefreshControl *refresh = [[UIRefreshControl alloc] init];
    
    refresh.attributedTitle = [[NSAttributedString alloc] initWithString:@"Pull to Load"];
    
    [refresh addTarget:self action:@selector(refresh)
     
      forControlEvents:UIControlEventValueChanged];

    self.refreshControl = refresh;
    
    DetailViewManager *detailViewManager = (DetailViewManager*)self.splitViewController.delegate;
    _openingViewController = (OpeningViewController*)detailViewManager.detailViewController;
}
- (void)viewWillAppear:(BOOL)animated  {
    
    FLOG(@" called");
    
    if ([[UIDevice currentDevice] userInterfaceIdiom] == UIUserInterfaceIdiomPad) {
        // Set to the opening view controller when we are shown
        DetailViewManager *detailViewManager = (DetailViewManager*)self.splitViewController.delegate;
        
        [detailViewManager showOpeningViewController];
        
    }
}
- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

- (void)stopRefresh {
    
    [self.refreshControl endRefreshing];
    
}
- (void)refresh {
    //FLOG(@"refresh called");
    [[OSCDStackManager sharedManager] loadDataInBackground];
    [self performSelector:@selector(stopRefresh) withObject:nil afterDelay:0.5];
}
- (void)loadMenuData {
    //FLOG(@" called");
    
    sectionMenus = [[NSMutableDictionary alloc] init];
    sectionsArray = [[NSArray alloc] initWithObjects:MENU_COMPANY_CONTACT, MENU_FINANCE, MENU_QUALITY, MENU_ADMIN, nil];
    
    [sectionMenus setObject:[[NSArray alloc] initWithObjects:SUBMENU_COMPANY, SUBMENU_PEOPLE, SUBMENU_CONTACTS, SUBMENU_PROJECTS, SUBMENU_TASKS, nil] forKey:MENU_COMPANY_CONTACT];
    [sectionMenus setObject:[[NSArray alloc] initWithObjects:@"Invoices", @"Payments", nil] forKey:MENU_FINANCE];
    [sectionMenus setObject:[[NSArray alloc] initWithObjects:@"Issues", @"Risks", nil] forKey:MENU_QUALITY];
    [sectionMenus setObject:[[NSArray alloc] initWithObjects:SUBMENU_BACKUPS, SUBMENU_BACKGROUND, SUBMENU_ICLOUD, nil] forKey:MENU_ADMIN];

}
-(void)storeChanged {
    //FLOG(@"storeChanged called");
    _isLoaded = YES;
    [(UITableView*)self.view reloadData];
}
#pragma mark - Table view data source

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
    //FLOG(@" called");
    return [sectionsArray count];
}
- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    //FLOG(@" called");
    
    return [[sectionMenus objectForKey:[sectionsArray objectAtIndex:section]] count];
}
- (NSString *)tableView:(UITableView *)tableView titleForHeaderInSection:(NSInteger)section {
    //FLOG(@" called");
    return [[sectionsArray objectAtIndex:section] description];
}
- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    static NSString *simpleTableIdentifier = @"SimpleTableItem";
    static NSString *inactiveTableIdentifier = @"InactiveTableItem";
    UITableViewCell *cell;
    NSObject *menuItem = [[sectionMenus objectForKey:[sectionsArray objectAtIndex:[indexPath section]]] objectAtIndex:[indexPath row]];
    NSString *menu = [menuItem description];
    
    if ([self isDisabled:menu] || !_isLoaded)
    {
        cell = [tableView dequeueReusableCellWithIdentifier:inactiveTableIdentifier];
        
        if (cell == nil) {
            cell = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:inactiveTableIdentifier];
        }
        
        cell.selectionStyle = UITableViewCellSelectionStyleNone;
        cell.textLabel.textColor = [UIColor grayColor];
        cell.backgroundView.backgroundColor = [UIColor lightGrayColor];
        
        if (!_isLoaded) {
            cell.accessoryType = UITableViewCellAccessoryNone;
            UIActivityIndicatorView *progressView = [[UIActivityIndicatorView alloc] initWithActivityIndicatorStyle:UIActivityIndicatorViewStyleGray];
            cell.accessoryView = progressView;
            [progressView startAnimating];
        }
        
    }
    else
    {
        cell = [tableView dequeueReusableCellWithIdentifier:simpleTableIdentifier];
        
        if (cell == nil) {
            cell = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:simpleTableIdentifier];
        }
        
        cell.textLabel.textColor = [UIColor blackColor];
        
    }
    
    cell.textLabel.text = menu;
    
    return cell;
}
- (NSIndexPath *)tableView:(UITableView *)tableView willSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
    NSString *menu = [[[sectionMenus objectForKey:[sectionsArray objectAtIndex:[indexPath section]]] objectAtIndex:[indexPath row]] description];
    if ([self isDisabled:menu] || !_isLoaded)
        return nil;
    else
        return indexPath;
    
}
- (BOOL)isDisabled:(NSString*)menu {
    
    if (!([menu isEqualToString:SUBMENU_COMPANY] ||
        [menu isEqualToString:SUBMENU_PEOPLE] ||
        [menu isEqualToString:SUBMENU_BACKUPS] ||
        [menu isEqualToString:SUBMENU_BACKGROUND])
        )
        return YES;
    else
        return NO;
    
}
- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
    //FLOG(@" called");
    NSObject *menu = [[sectionMenus objectForKey:[sectionsArray objectAtIndex:[indexPath section]]] objectAtIndex:[indexPath row]];
    
    NSString *menuString = menu.description;
    
    if ([menuString isEqualToString:SUBMENU_COMPANY]) {
        
        CompanyViewController *newTableViewController = [self.storyboard instantiateViewControllerWithIdentifier:@"CompanyViewController"];
        [self.navigationController pushViewController:newTableViewController animated:YES];
        return;
    }
    
    if ([menuString isEqualToString:SUBMENU_PEOPLE]) {
        
        PeopleViewController *newTableViewController = [self.storyboard instantiateViewControllerWithIdentifier:@"PeopleViewController"];
        [self.navigationController pushViewController:newTableViewController animated:YES];
        return;
    }
    if ([menuString isEqualToString:SUBMENU_BACKUPS]) {
        
        BackupsTableViewController *newTableViewController = [self.storyboard instantiateViewControllerWithIdentifier:@"BackupsTableViewController"];
        [self.navigationController pushViewController:newTableViewController animated:YES];
        return;
    }
    if ([menuString isEqualToString:SUBMENU_BACKGROUND]) {
        
        // if iPad set the detailViewController
        
        BackgroundTaskViewController *newViewController = [[BackgroundTaskViewController alloc] initWithNibName:@"BackgroundTaskViewController" bundle:nil];
        
        if ([[UIDevice currentDevice] userInterfaceIdiom] == UIUserInterfaceIdiomPad) {
            DetailViewManager *detailViewManager = (DetailViewManager*)self.splitViewController.delegate;
            FLOG(@" detailViewManager is %@", detailViewManager);
            
            // Create and configure a new detail view controller appropriate for the selection.
            
            detailViewManager.detailViewController = newViewController;
            
        }
        else {
            [self.navigationController pushViewController:newViewController animated:YES];
        }
        return;
    }
    ////FLOG(@" Menu %@ selected.", [menu description]);
    //[self performSegueWithIdentifier:[menu description] sender:tableView];
    
}

#pragma mark - Navigation

// In a storyboard-based application, you will often want to do a little preparation before navigation
- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender
{
    //FLOG(@" called");
    // Get the new view controller using [segue destinationViewController].
    // Pass the selected object to the new view controller.
    if ([[segue identifier] isEqualToString:@"Companies"]) {
        //[[segue destinationViewController] setDetailItem:];
    }

}


@end
