//
//  PeopleDetailViewController.m
//  CoreDataLibraryApp
//
//  Created by Duncan Groenewald on 13/02/2014.
//  Copyright (c) 2014 OSSH Pty Ltd. All rights reserved.
//

#import "PeopleDetailViewController.h"

@interface PeopleDetailViewController ()

@end

@implementation PeopleDetailViewController

- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil
{
    self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
    if (self) {
        // Custom initialization
        self.navigationBar.topItem.title = NSLocalizedString(@"", @"");
    }
    return self;
}

// -------------------------------------------------------------------------------
//	setNavigationPaneBarButtonItem:
//  Custom implementation for the navigationPaneBarButtonItem setter.
//  In addition to updating the _navigationPaneBarButtonItem ivar, it
//  reconfigures the navigationBar to either show or hide the
//  navigationPaneBarButtonItem.
// -------------------------------------------------------------------------------
- (void)setNavigationPaneBarButtonItem:(UIBarButtonItem *)navigationPaneBarButtonItem
{
    FLOG(@" called with %@", navigationPaneBarButtonItem);
    
    if (navigationPaneBarButtonItem != _navigationPaneBarButtonItem) {
        // Add the popover button to the left navigation item.
        [self.navigationBar.topItem setLeftBarButtonItem:navigationPaneBarButtonItem
                                                animated:NO];
        
        _navigationPaneBarButtonItem = navigationPaneBarButtonItem;
    }
}

#pragma mark - Managing the detail item

- (void)setDetailItem:(id)newDetailItem
{
    if (_detailItem != newDetailItem) {
        _detailItem = newDetailItem;
        
        // Update the view.
        [self configureView];
    }
    
}

- (void)configureView
{
    // Setup UI Controls
    [self.cancelButton setEnabled:NO];
    [self.cancelButton addTarget:self action:@selector(cancel) forControlEvents:UIControlEventTouchDown];
    [self.saveButton setEnabled:NO];
    [self.saveButton addTarget:self action:@selector(saveDetails) forControlEvents:UIControlEventTouchDown];
    
    // Set textField delegate
    self.detailDescriptionTextField.delegate = self;
        
    // Display the objects details
    [self displayDetails];
    
}

- (void)displayDetails
{
    // Update the user interface for the detail item.
    NSDateFormatter *dateFormatter = [[NSDateFormatter alloc] init];
    [dateFormatter setDateFormat:@"yyyyMMddHHmmssSSS"];
        
    if (self.detailItem) {
        self.detailDescriptionTextField.text = [[self.detailItem valueForKey:@"name"] description];
        self.imageView.image = [self.detailItem valueForKey:@"image"];
        self.timestampTextField.text = [dateFormatter stringFromDate:[self.detailItem valueForKey:@"timestamp"]];
    }
}

- (void)saveDetails
{
    // Save any changes in the UI components to the managedObject (can't cancel after this)
    
    if (self.detailItem) {
        
        [self.detailItem setValue:self.detailDescriptionTextField.text forKey:@"name"];
        [self.detailItem setValue:self.imageView.image forKey:@"image"];
        
        NSError *error;
        if (![[self.detailItem managedObjectContext] save:&error]) {
            FLOG(@" error saving changes!");
            FLOG(@" error is %@, %@", error, error.userInfo);
        } else {
            FLOG(@" changes saved!");
            [self setEdited:NO];
        }
    }
    
}

- (void)cancel
{
    // Update the user interface with the original data
    
    [self displayDetails];
    
    [self setEdited:NO];
    
}

- (void)setEdited:(BOOL)edited
{
    _edited = edited;
    
    [self.cancelButton setEnabled:edited];
    [self.saveButton setEnabled:edited];
}

- (void)refreshUI {
    FLOG(@"refreshUI called");
    [[NSOperationQueue mainQueue] addOperationWithBlock:^ {
        //Your code goes in here
        FLOG(@" about to call displayItem");
        [self displayDetails];
        FLOG(@" displayItem has been called");
    }];
}

- (void)viewDidLoad
{
    [super viewDidLoad];
	// Do any additional setup after loading the view, typically from a nib.
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(refreshUI) name:OSDataUpdatedNotification
                                               object:nil];
    self.title = NSLocalizedString(@"Person Detail", @"Person Detail");
    [self configureView];
}

- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

#pragma mark - TextFieldDelgate


- (BOOL)textField:(UITextField *)textField shouldChangeCharactersInRange:(NSRange)range replacementString:(NSString *)string
{
    [self setEdited:YES];
    return YES;
}

@end
